@@ -1,12 +1,12 @@
 #' Create simulated residuals
 #' 
 #' The function creates scaled residuals by simulating from the fitted model
-#' @param fittedModel fitted model object. Supported are generalized linear mixed models from 'lme4' (classes 'lmerMod', 'glmerMod'), generalized additive models ('gam' from 'mgcv', excluding extended families from 'mgcv'), 'glm' (including 'negbin' from 'MASS', but excluding quasi-distributions) and 'lm' model classes. 
-#' @param n integer number > 1, number of simulations to run. If possible, set to at least 250, better 1000. Smaller number > 50 can be chose if runtime is prohbitie, but discretization artefacts can occur at some point. 
+#' @param fittedModel a fitted model  of a class supported by DHARMa
+#' @param n number of simulations. Default is 100. A more save value would be 250 or even 1000. The smaller the number, the higher the stochastic error on the residuals. Also, for very small n, discretization artefacts can influence the tests. 
 #' @param refit if F, new data will be simulated and scaled residuals will be created by comparing observed data with new data. If T, the model will be refit on the simulated data (parametric bootstrap), and scaled residuals will be created by comparing observed with refitted residuals.
 #' @param integerResponse if T, noise will be added at to the residuals to maintain a uniform expectations for integer responses (such as Poisson or Binomial). Usually, the model will automatically detect the appropriate setting, so there is no need to adjust this setting.
 #' @param plot if T, \code{\link{plotSimulatedResiduals}} will be directly run after the simulations have terminated
-#' @param ... parameters to pass to the simulate function of the model object. An important use of this is to specify whether simulations should be conditional on the current random effect estimates. See details.
+#' @param ... parameters to pass to the simulate function of the model object. An important use of this is to specify whether simulations should be conditional on the current random effect estimates. See also details
 #' @param seed the random seed. The default setting, recommended for any type of data analysis, is to reset the random number generator each time the function is run, meaning that you will always get the same result when running the same code. NULL = no new seed is set, but previous random state will be restored after simulation. F = no seed is set, and random state will not be restored. The latter two options are only recommended for simulation experiments. See vignette for details.
 #' @return An S3 class of type "DHARMa", essentially a list with various elements. Implemented S3 functions include plot, print and \code{\link{residuals.DHARMa}}. Residuals returns the calculated scaled residuals, which can also be accessed via $scaledResiduals. The returned object additionally contains an element 'scaledResidualsNormal', which contains the scaled residuals transformed to a normal distribution (for stability reasons not recommended)
 #' 
@@ -14,11 +14,13 @@
 #' 
 #' \strong{Re-simulating random effects / hierarchical structure}: the first is that in a hierarchical model, several layers of stochasticity are aligned on top of each other. Specifically, in a GLMM, we have a lower level stochastic process (random effect), whose result enters into a higher level (e.g. Poisson distribution). For other hierarchical models such as state-space models, similar considerations apply. When simulating, we have to decide if we want to re-simulate all stochastic levels, or only a subset of those. For example, in a GLMM, it is common to only simulate the last stochastic level (e.g. Poisson) conditional on the fitted random effects. 
 #' 
-#' For controlling how many levels should be re-simulated, the simulateResidual function allows to pass on parameters to the simulate function of the fitted model object. Please refer to the help of the different simulate functions (e.g. ?simulate.merMod) for details. For merMod (lme4) model objects, the relevant parameters are parameters are use.u, and re.form
+#' For controlling how many levels should be re-simulated, the simulateResidual function allows to pass on parameters to the simulate function of the fitted model object. Please refer to the help of the different simulate functions (e.g. ?simulate.merMod) for details. For merMod (lme4) model objects, the relevant parameters are parameters are use.u and re.form
 #' 
 #' If the model is correctly specified, the simulated residuals should be flat regardles how many hierarchical levels we re-simulate. The most thorough procedure would therefore be to test all possible options. If testing only one option, I would recommend to re-simulate all levels, because this esentially tests the model structure as a whole. This is the default setting in the DHARMa package. A potential drawback is that re-simulating the lower-level random effects creates more variability, which may reduce power for detecing problems in the upper-level stochatic processes. 
 #' 
-#' \strong{Integer responses}: a second complication is the treatment of inter responses. Imaging we have observed a 0, and we predict 30\% zeros - what is the quantile that we should display for the residual? To deal with this problem and maintain a unifor response, the option integerResponse adds a uniform noise from -0.5 to 0.5 on the simulated and observed response. Note that this works because the expected distribution of this is flat - you can see this via hist(ecdf(runif(10000))(runif(10000))) 
+#' \strong{Integer responses}: a second complication is the treatment of inter responses. Imaging we have observed a 0, and we predict 30\% zeros - what is the quantile that we should display for the residual? To deal with this problem and maintain a uniform response, the option integerResponse adds a uniform noise from -0.5 to 0.5 on the simulated and observed response, which creates a uniform distribution - you can see this via hist(ecdf(runif(10000))(runif(10000))).
+#' 
+#'  DHARMa will try to automatically if the fitted model has an integer or discrete distribution via the family argument. However, in some cases the family does not allow to uniquely identify the distribution type. For example, a tweedie distribution can be inter or continous. Therefore, DHARMa will additionally check the simulation results for repeated values, and will change the distribution type if repeated values are found (a message is displayed in this case). 
 #' 
 #' \strong{Refitting or not}: a third issue is how residuals are calculated. simulateResiduals has two options that are controlled by the refit parameter:
 #' 
@@ -62,13 +64,13 @@ simulateResiduals <- function(fittedModel, n = 250, refit = F, integerResponse =
   out$nObs = nobs(fittedModel)
   out$nSim = n
   out$refit = refit
-  out$observedResponse = model.frame(fittedModel)[,1] 
+  out$observedResponse = getResponse(fittedModel) 
   
   # TODO - check if that works 
   nKcase = is.matrix(out$observedResponse)
   if(nKcase){
-    if(! (family$family %in% c("binomial", "betabinomial"))) securityAssertion("nKcase")
-    if(! (ncol(out$observedResponse)==2)) securityAssertion("nKcase")
+    if(! (family$family %in% c("binomial", "betabinomial"))) securityAssertion("nKcase - wrong family")
+    if(! (ncol(out$observedResponse)==2)) securityAssertion("nKcase - wrong dimensions of response")
     out$observedResponse = out$observedResponse[,1]
   }
 
@@ -84,6 +86,8 @@ simulateResiduals <- function(fittedModel, n = 250, refit = F, integerResponse =
     
   if(out$modelClass %in% c("glmmTMB")){
     out$fittedPredictedResponse = predict(fittedModel, type = "response") 
+  }else if(out$modelClass %in% c("HLfit")){
+    out$fittedPredictedResponse = predict(fittedModel, type = "response", re.form = ~0)[,1L] 
   }else{
     out$fittedPredictedResponse = predict(fittedModel, type = "response", re.form = ~0) 
   }
@@ -93,7 +97,7 @@ simulateResiduals <- function(fittedModel, n = 250, refit = F, integerResponse =
   
   ######## simulations ##################
   
-  simulations = simulate(fittedModel, nsim = n, ...)
+  simulations = getSimulations(fittedModel, nsim = n, ...)
   
   if(out$modelClass == "glmmTMB"){
     if(ncol(simulations) == 2*n){
@@ -182,7 +186,6 @@ simulateResiduals <- function(fittedModel, n = 250, refit = F, integerResponse =
   
   out$scaledResidualsNormal = qnorm(out$scaledResiduals + 0.00 )
 
-  
   out$time = proc.time() - ptm
   out$randomState = randomState
 
@@ -193,11 +196,13 @@ simulateResiduals <- function(fittedModel, n = 250, refit = F, integerResponse =
   return(out)
 }
 
-getPossibleModels<-function()c("lm", "glm", "negbin", "lmerMod", "glmerMod", "gam", "glmmTMB") 
+getPossibleModels<-function()c("lm", "glm", "negbin", "lmerMod", "glmerMod", "gam", "bam", "glmmTMB", "HLfit") 
 
 checkModel <- function(fittedModel){
   if(!(class(fittedModel)[1] %in% getPossibleModels())) warning("DHARMa: fittedModel not in class of supported models. Absolutely no guarantee that this will work!")
   
+  if (class(fittedModel)[1] == "glmmTMB") message("It seems you are diagnosing a glmmTBM model. There are still a few minor limitations associatd with this package. The most important is that glmmTMB doesn't implement an option to create unconditional predictions from the model, which means that predicted values (in res ~ pred) plots include the random effects. With strong random effects, this can sometimes create diagonal patterns from bottom left to top right in the res ~ pred plot. All other tests and plots should work as desired. Please see https://github.com/florianhartig/DHARMa/issues/16 for further details.")
+
   if (class(fittedModel)[1] == "gam" ) if (class(fittedModel$family)[1] == "extended.family") stop("It seems you are trying to fit a model from mgcv that was fit with an extended.family. Simulation functions for these families are not yet implemented in DHARMa. See issue https://github.com/florianhartig/DHARMa/issues/11 for updates about this")
 
 }
@@ -205,78 +210,20 @@ checkModel <- function(fittedModel){
 
 getFixedEffects <- function(fittedModel){
   
-  if(class(fittedModel)[1] %in% c("glm", "lm", "gam", "negbin") ){
+  if(class(fittedModel)[1] %in% c("glm", "lm", "gam", "bam", "negbin") ){
     out  = coef(fittedModel)
-  } else if(class(fittedModel)[1] %in% c("glmerMod", "lmerMod")){
-    out = lme4::fixef(fittedModel)
+  } else if(class(fittedModel)[1] %in% c("glmerMod", "lmerMod", "HLfit")){
+    out = fixef(fittedModel)
   } else if(class(fittedModel)[1] %in% c("glmmTMB")){
     out = glmmTMB::fixef(fittedModel)
     out = out$cond
-  } else stop()
+  } else {
+    out = coef(fittedModel)
+    if(is.null(out)) out = fixef(fittedModel)
+  }
   return(out)
 }
 
-
-#' @importFrom lme4 refit
-NULL
-
-
-#' Refit a Model with a Different Response
-#' 
-#' @param object a fitted model
-#' @param newresp a new response
-#' @param ... further arguments, no effect implemented for this S3 class
-#' @example inst/examples/helpRefit.R
-#' @export
-refit.lm <- function(object, newresp, ...){
-  
-  newData <-model.frame(object)  
-
-  if(is.vector(newresp)){
-    newData[,1] = newresp
-  } else if (is.factor(newresp)){
-    # Hack to make the factor binomial case work
-    newData[,1] = as.numeric(newresp) - 1
-  } else {
-    # Hack to make the binomial n/k case work
-    newData[[1]] = NULL
-    newData = cbind(newresp, newData)
-  }     
-  
-  refittedModel = update(object, data = newData)
-  return(refittedModel)
-}
-
-#' Refit a Model with a Different Response
-#' 
-#' @param object a fitted model
-#' @param newresp a new response
-#' @param ... further arguments, no effect implemented for this S3 class
-#' @example inst/examples/helpRefit.R
-#' @export
-refit.glmmTMB <- function(object, newresp, ...){
-  
-  newData <-model.frame(object)  
-  
-  matrixResp = is.matrix(newData[[1]])
-
-  if(matrixResp & !is.null(ncol(newresp))){
-    # Hack to make the factor binomial case work
-    tmp = colnames(newData[[1]])
-    newData[[1]] = NULL
-    newData = cbind(newresp, newData)
-    colnames(newData)[1:2] = tmp
-  } else if(!is.null(ncol(newresp))){
-    newData[[1]] = newresp[,1]
-  } else {
-    newData[[1]] = newresp 
-  }
-
-  refittedModel = update(object, data = newData)
-  return(refittedModel)
-}
-
-
 securityAssertion <- function(context = "Not provided", stop = F){
   generalMessage = "Message from DHARMa package: a security assertion was not met. This means that during the execution of a DHARMa function, some unexpected conditions ocurred. Even if you didn't get an error, your results may not be reliable. Please check with the help if you use the functions as intended. If you think that the error is not on your side, I would be grateful if you could report the problem at https://github.com/florianhartig/DHARMa/issues \n\n Context:"
   if (stop == F) warning(paste(generalMessage, context))  
@@ -301,6 +248,7 @@ recalculateResiduals <- function(simulationOutput, group = NULL, aggregateBy = s
   if(!is.null(simulationOutput$original)) simulationOutput = simulationOutput$original
 
   out = list()
+  out$original = simulationOutput
   
   if(is.null(group)) return(simulationOutput)
   else group =as.factor(group)
@@ -311,14 +259,11 @@ recalculateResiduals <- function(simulationOutput, group = NULL, aggregateBy = s
   out$observedResponse = aggregateByGroup(simulationOutput$observedResponse)
   out$fittedPredictedResponse = aggregateByGroup(simulationOutput$fittedPredictedResponse)
   out$simulatedResponse = apply(simulationOutput$simulatedResponse, 2, aggregateByGroup)
-  out$scaledResiduals = rep(NA, out$nGroups)
-
+  
   if (simulationOutput$refit == F){
-    if(simulationOutput$integerResponse == T){
-      for (i in 1:out$nGroups) out$scaledResiduals[i] <- ecdf(out$simulatedResponse[i,] + runif(out$nGroups, -0.5, 0.5))(out$observedResponse[i] + runif(1, -0.5, 0.5))
-    } else {
-      for (i in 1:out$nGroups) out$scaledResiduals[i] <- ecdf(out$simulatedResponse[i,])(out$observedResponse[i])
-    } 
+    
+    out$scaledResiduals = getQuantile(simulations = out$simulatedResponse , observed = out$observedResponse , n = out$nGroups, nSim = simulationOutput$nSim, integerResponse = simulationOutput$integerResponse)
+ 
   ######## refit = T ##################   
   } else {
 
@@ -326,60 +271,17 @@ recalculateResiduals <- function(simulationOutput, group = NULL, aggregateBy = s
     out$fittedResiduals = aggregateByGroup(simulationOutput$fittedResiduals)
     out$refittedResiduals = apply(simulationOutput$refittedResiduals, 2, aggregateByGroup)
     out$refittedPearsonResiduals = apply(simulationOutput$refittedPearsonResiduals, 2, aggregateByGroup)
-    
-    if(simulationOutput$integerResponse == T){
-      for (i in 1:out$nGroups) out$scaledResiduals[i] <- ecdf(out$refittedResiduals[i,] + runif(out$nGroups, -0.5, 0.5))(out$fittedResiduals[i] + runif(1, -0.5, 0.5))
-    } else {
-      for (i in 1:out$nGroups) out$scaledResiduals[i] <- ecdf(out$refittedResiduals[i,])(out$fittedResiduals[i])
-    } 
+  
+    out$scaledResiduals = getQuantile(simulations = out$refittedResiduals , observed = out$fittedResiduals , n = out$nGroups, nSim = simulationOutput$nSim, integerResponse = simulationOutput$integerResponse)
+
   }
+  
   # hack - the c here will result in both old and new outputs to be present resulting output, but a named access should refer to the new, grouped calculations
+  # question to myself - what's the use of that, why not erase the old outputs? they are anyway saved in the old object
+  
   out$aggregateByGroup = aggregateByGroup
   out = c(out, simulationOutput)
-  out$original = simulationOutput
   class(out) = "DHARMa"
   return(out)
 }
   
-
-getQuantile <- function(simulations, observed, n, nSim, integerResponse){
-  
-  scaledResiduals = rep(NA, n)
-  
-  if(integerResponse == F){
-    
-    values = c(as.vector(t(simulations)), observed)
-    
-    if(any (duplicated(values))){
-      integerResponse = T
-      # repeated = unique(values[which(duplicated(values))])      
-      if(any(integerResponse)) message("Model family was recoginzed or set as continous, but duplicate values were detected in the simulation - changing to integer residuals (see help for details)")
-    } 
-  } 
-
-  for (i in 1:n){
-    if(integerResponse == T){
-      scaledResiduals[i] <- ecdf(simulations[i,] + runif(nSim, -0.5, 0.5))(observed[i] + runif(1, -0.5, 0.5))
-
-    }else{
-      scaledResiduals[i] <- ecdf(simulations[i,])(observed[i])
-    }
-  }
-  return(scaledResiduals)
-}
-
-# 
-# 
-# testData = createData(sampleSize = 200, family = gaussian(),
-#                       randomEffectVariance = 0, numGroups = 5)
-# fittedModel <- glmmTMB(observedResponse ~ Environment1,
-#                    data = testData)
-# simulationOutput <- simulateResiduals(fittedModel = fittedModel)
-# 
-# sims = simulationOutput$simulatedResponse
-# sims[1, c(1,6,8)] = 0
-# any(apply(sims, 1, anyDuplicated))
-# getQuantile(simulations = sims, observed = testData$observedResponse, n = 200, integerResponse = F, nSim = 250)
-# 
-# 
-# 
